@@ -148,13 +148,13 @@ class Platform(Enum):
     ANDROID     = 'AND'
 
 
-class ProfileSearchPlatform(Enum):
+class UserSearchPlatform(Enum):
     EPIC_GAMES  = 'epic'
     PLAYSTATION = 'psn'
     XBOX        = 'xbl'
 
 
-class ProfileSearchMatchType(Enum):
+class UserSearchMatchType(Enum):
     EXACT = 'exact'
     PREFIX = 'prefix'
 
@@ -166,9 +166,9 @@ class ReadyState(Enum):
 
 
 class AwayStatus(Enum):
-    ONLINE        = PresenceShow.NONE
-    AWAY          = PresenceShow.AWAY
-    EXTENDED_AWAY = PresenceShow.EXTENDED_AWAY
+    ONLINE        = None
+    AWAY          = 'away'
+    EXTENDED_AWAY = 'xa'
 
 
 class SeasonStartTimestamp(Enum):
