@@ -66,6 +66,9 @@ class Avatar:
     background: Optional[Union[List[:class:`str`], :class:`KairosBackgroundColorPreset`]]
         A list of exactly three hex color values represented as strings.
     """  # noqa
+
+    __slots__ = ('asset', 'background_colors')
+
     def __init__(self, *, asset: Optional[str] = None,
                  background_colors: ListOrPreset = None) -> None:
         self.asset = asset
@@ -79,6 +82,12 @@ class Avatar:
         return ('<Avatar asset={0.asset!r} '
                 'background_colors={0.background_colors!r}>'.format(self))
 
+    def __eq__(self, other):
+        return isinstance(other, Avatar) and other.to_dict() == self.to_dict()
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def to_dict(self) -> Dict[str, Any]:
         """Converts it into a fortnite friendly dict.
 
